import os
from payjs import PayJS

MCHID = os.environ.get('MCHID')
KEY = os.environ.get('KEY')

# 初始化
p = PayJS(MCHID, KEY)

# 扫码支付
OUT_TRADE_NO = '2017TEST'     # 外部订单号（自己的支付系统的订单号，请保证唯一）
TOTAL_FEE = 1                 # 支付金额，单位为分，金额最低 0.01 元最多 10000 元
BODY = '测试支付'              # 订单标题
ATTACH = 'info'
r = p.QRPay(out_trade_no=OUT_TRADE_NO, total_fee=TOTAL_FEE, body=BODY, attach=ATTACH)
if r:
    print(r.code_url)         # 二维码地址（weixin:// 开头，请使用此地址构建二维码）
    print(r.qrcode)           # 二维码地址（https:// 开头，为二维码图片的地址）
    print(r.payjs_order_id)   # 订单号（PAYJS 的）
else:
    print(r.STATUS_CODE)      # HTTP 请求状态码
    print(r.ERROR_NO)         # 错误码
    print(r.error_msg)        # 错误信息
    print(r)

# 收银台支付
c = p.CashierPay(out_trade_no=OUT_TRADE_NO, total_fee=TOTAL_FEE, body=BODY)
if c:
    print(c.redirect)         # 要跳转到的收银台网址
else:
    print(c.error_msg)        # 错误信息
    print(c)

# 订单查询
s = p.check_status(payjs_order_id=r.payjs_order_id)
if s:
    print(s.paid)            # 是否已支付
else:
    print(s.error_msg)       # 错误信息
    print(s)

# 订单关闭
t = p.close(r.payjs_order_id)
if t:
    print('Success')
else:
    print('Error')
    print(t.return_msg)



class PayJSNotify:
    # # <QueryDict: {'attach': '', 'mchid': ['WDQGQD'], 'openid': ['o7LFAwWqr1gw2Hh6Z_Uj1XFgDE0M'], 'out_trade_no': ['84post::6b0a44c787c646b1bb4c5125'], 'payjs_order_id': ['2018052711173900160753334'], 'return_code': ['1'], 'time_end': ['2018-05-27 11:17:50'], 'total_fee': ['1200'], 'transaction_id': ['4200000158201805272545912299'], 'sign': ['0AEC12516865689A1A786E131D4802EA']}>
    def __init__(self, notify_content):
        if type(notify_content) is str:
            from urllib import parse
            notify = dict(parse.parse_qsl(notify_content))
        else:
            notify = dict(notify_content)



import json
import logging
import requests

from pprint import pformat

from payjs.result import PayJSResultSuccess, PayJSResultFail
from payjs.sign import get_signature, check_signature
from payjs.utils import check_url
from payjs.exceptions import InvalidSignatureException, InvalidInfoException

logger = logging.getLogger(__name__)


class PayJS:
    """
    PayJS
    用于 payjs.cn 的 API
    """

    FORCE_SSL = True

    payjs_order_id = ''

    def __init__(self, mchid: str, key: str, notify_url=None, **kwargs):
        """
        初始化

        :param mchid: 商户号
        :param key: 密钥
        :param notify_url: （可选）异步通知的 URL，留空为不通知或在发起请求时设置
        :param FORCE_SSL: （默认为 True）回调地址强制使用 HTTPS
        """

        for k, v in kwargs.items():
            self.__setattr__(k, v)

        if type(mchid) is not str:
            raise InvalidInfoException(-2001, "商户号格式必须为字符串")
        if len(mchid) != 6:
            logger.warning('商户号可能有误（不是六位）')

        self.mchid = mchid

        if type(key) is not str:
            raise InvalidInfoException(-2002, "密钥格式必须为字符串")
        if len(key) != 16:
            logger.warning('密钥可能有误（不是 16 位）')

        self.key = key

        if not check_url(notify_url, force_ssl=self.FORCE_SSL):
            raise InvalidInfoException(-2003, "通知回调网址错误")

        self.notify_url = notify_url

    def request(self, url: str, data: dict):
        """
        处理请求（请求时会过滤值为空的参数）

        :param url: 请求的 url
        :param data: 请求的参数字典（不包含签名）
        :return: 返回一个 PayJSResultSuccess 或 PayJSResultFail 类元素
        """
        data['sign'] = get_signature(self.key, data)
        data = {k: v for k, v in data.items() if v}
        r = requests.post(url, data=data, allow_redirects=False)
        return self.parse_response(r)

    def parse_response(self, raw_response: requests.Response):
        """
        处理请求，将 requests 的返回包装为 PayJSResult
        :param raw_response: requests 的返回值
        :return: 一个 PayJSResult 元素（Success 或 Fail）
        """
        if raw_response.status_code == 200:
            # 扫码支付 与 订单查询；收银台支付失败
            try:
                j = json.loads(raw_response.content)
            except json.JSONDecodeError:
                return PayJSResultFail(raw_response=raw_response, r_json=False)
            except UnicodeDecodeError:
                return PayJSResultFail(raw_response=raw_response, r_json=False)

            try:
                check_signature(self.key, j)
            except InvalidSignatureException:
                response = PayJSResultFail(raw_response=raw_response, r_json=j)
                response.error_msg = '返回的签名错误'

            if str(j.get('return_code')) == '0':  # 请求失败
                response = PayJSResultFail(raw_response=raw_response, r_json=j)
            else:
                response = PayJSResultSuccess(raw_response=raw_response, r_json=j)

        elif raw_response.status_code == 302:
            # 收银台支付
            return PayJSResultSuccess(raw_response=raw_response, r_json=None)
        else:
            response = PayJSResultFail(raw_response=raw_response, r_json=False)
        return response

    def check_status_by_payjs_order_id(self, payjs_order_id=None):
        """
        通过 PayJS 订单号来查询交易状态
        :param payjs_order_id: PayJS 订单号
        :return: 返回一个 PayJSResult 成员，当其为 SUCCESS 时存在一个 PAID 属性来表明是否已支付
        """
        if not payjs_order_id:
            payjs_order_id = self.payjs_order_id
            if not payjs_order_id:
                raise InvalidInfoException(-3001, '无效的订单号（目前仅支持 PayJS 订单号查询）')

        payjs_order_id = str(payjs_order_id)

        if not 1 <= len(payjs_order_id) <= 32:
            logger.warning('订单号位数可能错误（需要在 1 - 32 位）')

        url = r'https://payjs.cn/api/check'

        data = {
            'payjs_order_id': payjs_order_id,
        }

        ret = self.request(url, data)

        # return self.request(url, data)
        return ret

    def check_status(self, *, payjs_order_id=None):
        return self.check_status_by_payjs_order_id(payjs_order_id)

    def native(self, total_fee: int, out_trade_no, body: str = '', notify_url=None, attach=None):
        """
        发起扫码支付
        :param total_fee: 支付金额，单位为分，介于 1 - 1000000 之间
        :param out_trade_no: 订单号，应保证唯一性，1-32 字符
        :param body: （可选）订单标题，0 - 32 字符
        :param notify_url: （可选）回调地址，留空使用默认，传入空字符串代表无需回调
        :param attach: （可选）用户自定义数据，在notify的时候会原样返回
        :return:
        """
        url = r'https://payjs.cn/api/native'

        if not total_fee > 0:
            raise InvalidInfoException(-2004, "金额必须为正整数（单位为分）")
        if not total_fee <= 100000:
            logger.warning("金额可能错误（单笔最高为 1000 元）")

        out_trade_no = str(out_trade_no)

        if not out_trade_no:
            logger.warning("用户端订单号不可省略")
        if not len(out_trade_no) <= 32:
            logger.warning("用户端订单号最多为 32 位")

        if not len(body) <= 32:
            logger.warning("标题最多为 32 位")

        if notify_url is None:
            notify_url = self.notify_url
        if not check_url(notify_url, force_ssl=self.FORCE_SSL):
            raise InvalidInfoException(-2003, '通知回调地址有误')

        data = {
            'mchid': self.mchid,
            'total_fee': total_fee,
            'out_trade_no': out_trade_no,
            'body': body,
            'notify_url': notify_url,
            'attach': attach,
        }

        ret = self.request(url, data)
        return ret

    def cashier(self, total_fee: int, out_trade_no, body: str = '', notify_url=None, callback_url=None, attach=None):
        """
        发起收银台支付

        注：目前此接口在参数传递错误时并不会有提示，依然会返回一个跳转后的网址
        :param total_fee: 支付金额，单位为分，介于 1 - 1000000 之间
        :param out_trade_no: 订单号，应保证唯一性，1-32 字符
        :param body: （可选）订单标题，0 - 32 字符
        :param notify_url: （可选）回调地址，留空使用默认，传入空字符串代表无需回调
        :param callback_url: （可选）（暂无效）支付成功后前端跳转地址
        :param attach: （可选）用户自定义数据，在notify的时候会原样返回
        :return: PayJSResult
        """
        url = r'https://payjs.cn/api/cashier'

        if not total_fee > 0:
            raise InvalidInfoException(-2004, "金额必须为正整数（单位为分）")
        if not total_fee <= 100000:
            logger.warning("金额可能错误（单笔最高为 1000 元）")

        out_trade_no = str(out_trade_no)

        if not out_trade_no:
            logger.warning("用户端订单号不可省略")
        if not len(out_trade_no) <= 32:
            logger.warning("用户端订单号最多为 32 位")

        if not len(body) <= 32:
            logger.warning("标题最多为 32 位")

        if notify_url is None:
            notify_url = self.notify_url
        if not check_url(notify_url, force_ssl=self.FORCE_SSL):
            raise InvalidInfoException(-2003, '通知回调地址有误')

        if not check_url(notify_url, force_ssl=self.FORCE_SSL):
            raise InvalidInfoException(-2004, '前端跳转地址有误')

        data = {
            'mchid': self.mchid,
            'total_fee': total_fee,
            'out_trade_no': out_trade_no,
            'body': body,
            'notify_url': notify_url,
            'callback_url': callback_url,
            'attach': attach
        }

        ret = self.request(url, data)
        return ret

    def close(self, payjs_order_id=None):
        """
        通过 PayJS 订单号来查询交易状态
        :param payjs_order_id: PayJS 订单号
        :return: 返回一个 PayJSResult 成员，当其为 SUCCESS 时存在一个 PAID 属性来表明是否已支付
        """
        if not payjs_order_id:
            payjs_order_id = self.payjs_order_id
            if not payjs_order_id:
                raise InvalidInfoException(-2005, '未提供订单号')

        payjs_order_id = str(payjs_order_id)

        if not 1 <= len(payjs_order_id) <= 32:
            logger.warning('订单号可能错误（位数需要在 1 - 32 位）')

        url = r'https://payjs.cn/api/close'

        data = {
            'payjs_order_id': payjs_order_id,
        }

        ret = self.request(url, data)

        # return self.request(url, data)
        return ret

    QRPay = native
    CashierPay = cashier



class PayJSException(Exception):

    def __init__(self, code, msg):
        """
        :param code: Error code
        :param msg: Error message
        """
        self.code = code
        self.msg = msg

    def __str__(self):
        return '[ERROR] Code: {code}, Message: {msg}'.format(
            code=self.code,
            msg=self.msg
        )

    def __repr__(self):
        return '{klass} Error ({code}:: {msg})'.format(
            klass=self.__class__.__name__,
            code=self.code,
            msg=self.msg
        )


class InvalidSignatureException(PayJSException):

    def __init__(self, code=-1001, msg='Invalid signature'):
        super().__init__(code, msg)


class InvalidInfoException(PayJSException):
    def __init__(self, code=-2000, msg='Invalid info'):
        super().__init__(code, msg)



import logging
from hashlib import md5
from urllib.parse import urlencode, unquote_plus

from payjs.exceptions import InvalidSignatureException

logger = logging.getLogger(__name__)


def get_signature(key: str, data: dict):
    """
    签名过程如下：
    0. 忽略已经存在的 sign
    1. 将所有参数名按照字典序排列，并以 "参数1=值1&参数2=值2&参数3=值3" 的形式排序
    2. 将上面的内容加上 "&key=KEY"（KEY 为设置的商户密钥）
    3. 将组合后的字符串转换为大写 MD5

    :param key: 商户密钥
    :param data: 要签名的参数字典
    :return: 签名后的字符串
    :rtype: str
    """
    d = data.copy()

    # pop 掉 sign 字段
    try:
        d.pop('sign')
    except KeyError:
        pass

    # pop 掉无效字段
    p = sorted([x for x in d.items() if (x[1] or x[1] == 0)], key=lambda x: x[0])

    p.append(('key', key))

    p = unquote_plus(urlencode(p))

    h = md5()
    h.update(p.encode())

    r = h.hexdigest().upper()

    return r


def check_signature(key: str, data: dict, sign: str = None):
    """
       校验签名，签名不通过会抛出 InvalidSignature 异常
       :param key: 商户密钥
       :param data: 要签名的参数字典
       :param sign: 要校验的签名，省略（为 None）则从 data 中取 sign 字段
       :rtype: bool
       """
    if sign is None:
        sign = data.get('sign')

    if get_signature(key, data) == sign:
        return True

    raise InvalidSignatureException



import re


def check_url(url=None, force_ssl=True):
    """
    检测回调 URL 是否符合规范
    目前支持普通网址（纯网址必须以/结尾）、带端口的网址、纯 IP 地址、中文网址（需先进行 PunyCode 编码）

    默认只支持 https，要想支持 HTTP 请手动设置实例的 FORCE_SSL 为 False
    :param url: 要检测的网址
    :param force_ssl: 是否强制使用 https
    :return: 是否通过（空网址直接通过）
    """
    if not url:
        return True

    r_protocal = r'https' if force_ssl else r'http|https'
    r_url = r'(xn--)?[A-Za-z0-9\.]{1,}\.([A-Za-z]{2,6}|xn--[A-Za-z0-9]{1,})'
    r_ip = r'\.'.join([r'(1\d{2}|2[0-4]\d|25[0-5]|[1-9]\d|\d)'] * 4)
    r_port = r'\d{1,5}'
    r_path = r'[A-Za-z\.\/]*'

    r = r'^(?P<protocal>{}):\/\/((?P<url>{})|(?P<ip>{}))(:(?P<port>{}))?\/(?P<path>{})$'.format(r_protocal, r_url,
                                                                                                r_ip, r_port,
                                                                                                r_path)
    r = re.compile(r)
    if r.match(url):
        return True
    else:
        return False



import logging
from pprint import pformat

logger = logging.getLogger(__name__)


class PayJSResultBase:
    def __init__(self, raw_response, r_json: dict = None):
        self.raw_response = raw_response  # 原始 requests.Response 数据
        self.content = raw_response.content  # 原始 response 的 content
        self.url = raw_response.url  # 请求的 url
        self.STATUS_CODE = raw_response.status_code  # HTTP 请求返回的状态吗
        self.json = r_json  # 请求返回的内容包装后的 JSON 值（以 dict 存储或为 None/False）
        self.JSON = self.json

    def __repr__(self):
        d = self.__dict__.copy()
        for m in ('key', 'content', 'raw_response', 'ERROR', 'JSON', 'ERROR_MSG'):
            try:
                d.pop(m)
            except KeyError:
                continue
        return pformat(d)


class PayJSResultSuccess(PayJSResultBase):
    def __bool__(self):
        return True

    def __init__(self, raw_response, **kwargs):
        super(PayJSResultSuccess, self).__init__(raw_response=raw_response, **kwargs)

        if type(self.json) is dict:
            for k, v in self.json.items():
                if k not in ['sign']:
                    setattr(self, k, v)

        if '/api/check' in self.url:
            self.PAID = True if getattr(self, 'status') == 1 else False  # 是否已支付
            self.paid = self.PAID
        if '/api/cashier' in self.url:
            self.REDIRECT = raw_response.headers.get('Location')
            self.redirect = self.REDIRECT


class PayJSResultFail(PayJSResultBase):
    ERROR_NO = 0

    def __bool__(self):
        return False

    def __init__(self, raw_response, **kwargs):
        super().__init__(raw_response=raw_response, **kwargs)
        if not self.json:
            self.error_msg = '请求失败'  # 错误信息
        else:
            self.error_msg = self.json.get('msg') or self.json.get('return_msg')



#!/usr/bin/env python
from setuptools import setup

VERSION = '1.0.0'
DOWNLOAD_URL = 'https://github.com/ImSingee/PayJS/archive/v{}.tar.gz'.format(VERSION)

setup(
    name='payjs',
    version=VERSION,
    packages=['payjs'],
    url='https://github.com/ImSingee/PayJS',
    license='MIT',
    author='Brian Wang',
    author_email='imsingee@gmail.com',
    description='An interface for payjs.cn',
    install_requires=[
        "requests>=2.18.4",
    ],
    keywords='python package payjs interface API wechat pay',
    download_url=DOWNLOAD_URL,
    classifiers=[
        "Programming Language :: Python :: 3.6",
    ]
)
